/* Teensyduino Core Library
 * http://www.pjrc.com/teensy/
 * Copyright (c) 2013 PJRC.COM, LLC.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * 1. The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * 2. If the Software is incorporated into a build system that allows
 * selection among a list of target devices, then similar target
 * devices manufactured by PJRC.COM must be included in the list of
 * target devices and selectable in the same manner.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef _core_pins_h_
#define _core_pins_h_

#include "pins_arduino.h"

#include "ShieldGPIOB.h"
#include "ShieldGPIOC.h"
#include "ShieldGPIOD.h"

#define HIGH		1
#define LOW		0
#define INPUT		0
#define OUTPUT		1
#define INPUT_PULLUP	2
#define LSBFIRST	0
#define MSBFIRST	1
#define _BV(n)		(1<<(n))
#define CHANGE		4
#define FALLING		2
#define RISING		3


/*
FRDM-K64

Pin - Connector - Grove Shield
---- 	------------------------
C16		RX	- UART
C17 	TX	- UART
B9 		D2	- D2
A1(C12)	D3	- D3 D2
B23		D4	- D4 D3
A2(C13)	D5	- D5 D4
C2		D6	- D6 D5
C3		D7	- D7 D6

A0(C15)	D8	- D8 D7
C4		D9	-    D8
D0		D10
D2		D11
D3		D12
D1		D13
(gnd)
(aref)
E25		SDA - I2C(x4)
E24		SCI	- I2C(x4)

--------------------
C10		A5	- P2
C11		A4	- A3
B11		A3	- A3 A2
B10		A2	- A2 A1
B3		A1	- A1 A0
B2		A0	- A0

*/

#define CORE_NUM_TOTAL_PINS     34
#define CORE_NUM_DIGITAL        34
#define CORE_NUM_INTERRUPT      34
#define CORE_NUM_ANALOG         21
#define CORE_NUM_PWM            11



#define	SHIELD_D2_PORT	ShieldGPIOB
#define	SHIELD_D3_PORT	ShieldGPIOC
#define	SHIELD_D4_PORT	ShieldGPIOB
#define	SHIELD_D5_PORT	ShieldGPIOC
#define	SHIELD_D6_PORT	ShieldGPIOC
#define	SHIELD_D7_PORT	ShieldGPIOC
#define	SHIELD_D8_PORT	ShieldGPIOC
#define	SHIELD_D9_PORT	ShieldGPIOC
#define	SHIELD_D10_PORT	ShieldGPIOD
#define	SHIELD_D11_PORT	ShieldGPIOD
#define	SHIELD_D12_PORT	ShieldGPIOD
#define	SHIELD_D13_PORT	ShieldGPIOD

#define	SHIELD_D2_PIN	ShieldD2
#define	SHIELD_D3_PIN	ShieldD3
#define	SHIELD_D4_PIN	ShieldD4
#define	SHIELD_D5_PIN	ShieldD5
#define	SHIELD_D6_PIN	ShieldD6
#define	SHIELD_D7_PIN	ShieldD7
#define	SHIELD_D8_PIN	ShieldD8
#define	SHIELD_D9_PIN	ShieldD9
#define	SHIELD_D10_PIN	ShieldD10
#define	SHIELD_D11_PIN	ShieldD11
#define	SHIELD_D12_PIN	ShieldD12
#define	SHIELD_D13_PIN	ShieldD13


#ifdef __cplusplus
extern "C" {
#endif

void digitalWrite(uint8_t pin, uint8_t val);
void digitalWriteFast(uint8_t pin, uint8_t val);

uint8_t digitalRead(uint8_t pin);
uint8_t digitalReadFast(uint8_t pin);

void pinMode(uint8_t pin, uint8_t mode);
void init_pins(void);
void analogWrite(uint8_t pin, int val);
void analogWriteRes(uint32_t bits);
static inline void analogWriteResolution(uint32_t bits) { analogWriteRes(bits); }
void analogWriteFrequency(uint8_t pin, float frequency);
void analogWriteDAC0(int val);
#ifdef __cplusplus
//void attachInterruptVector(IRQ_NUMBER_t irq, void (*function)(void));
#else
//void attachInterruptVector(enum IRQ_NUMBER_t irq, void (*function)(void));
#endif
void attachInterrupt(uint8_t pin, void (*function)(void), int mode);
void detachInterrupt(uint8_t pin);
void _init_Teensyduino_internal_(void);

int analogRead(uint8_t pin);
void analogReference(uint8_t type);
void analogReadRes(unsigned int bits);
static inline void analogReadResolution(unsigned int bits) { analogReadRes(bits); }
void analogReadAveraging(unsigned int num);
void analog_init(void);


#if defined(__MK20DX128__) || defined(__MK20DX256__)
#define DEFAULT         0
#define INTERNAL        2
#define INTERNAL1V2     2
#define INTERNAL1V1     2
#define EXTERNAL        0

#elif defined(__MKL26Z64__)
#define DEFAULT         0
#define INTERNAL        0
#define EXTERNAL        1
#endif


int touchRead(uint8_t pin);


static inline void shiftOut(uint8_t, uint8_t, uint8_t, uint8_t) __attribute__((always_inline, unused));
extern void _shiftOut(uint8_t dataPin, uint8_t clockPin, uint8_t bitOrder, uint8_t value) __attribute__((noinline));
extern void shiftOut_lsbFirst(uint8_t dataPin, uint8_t clockPin, uint8_t value) __attribute__((noinline));
extern void shiftOut_msbFirst(uint8_t dataPin, uint8_t clockPin, uint8_t value) __attribute__((noinline));

static inline void shiftOut(uint8_t dataPin, uint8_t clockPin, uint8_t bitOrder, uint8_t value)
{
        if (__builtin_constant_p(bitOrder)) {
                if (bitOrder == LSBFIRST) {
                        shiftOut_lsbFirst(dataPin, clockPin, value);
                } else {
                        shiftOut_msbFirst(dataPin, clockPin, value);
                }
        } else {
                _shiftOut(dataPin, clockPin, bitOrder, value);
        }
}

static inline uint8_t shiftIn(uint8_t, uint8_t, uint8_t) __attribute__((always_inline, unused));
extern uint8_t _shiftIn(uint8_t dataPin, uint8_t clockPin, uint8_t bitOrder) __attribute__((noinline));
extern uint8_t shiftIn_lsbFirst(uint8_t dataPin, uint8_t clockPin) __attribute__((noinline));
extern uint8_t shiftIn_msbFirst(uint8_t dataPin, uint8_t clockPin) __attribute__((noinline));

static inline uint8_t shiftIn(uint8_t dataPin, uint8_t clockPin, uint8_t bitOrder)
{
        if (__builtin_constant_p(bitOrder)) {
                if (bitOrder == LSBFIRST) {
                        return shiftIn_lsbFirst(dataPin, clockPin);
                } else {
                        return shiftIn_msbFirst(dataPin, clockPin);
                }
        } else {
                return _shiftIn(dataPin, clockPin, bitOrder);
        }
}

void _reboot_Teensyduino_(void) __attribute__((noreturn));
void _restart_Teensyduino_(void) __attribute__((noreturn));

void yield(void);

void delay(uint32_t msec);

extern volatile uint32_t systick_millis_count;
extern volatile uint32_t delay_millis_count;

static inline uint32_t millis(void) __attribute__((always_inline, unused));
static inline uint32_t millis(void)
{
	return systick_millis_count; // single aligned 32 bit is atomic;
}

uint32_t micros(void);
void delayMicroseconds(uint32_t usec);

extern uint8_t pwmStep;
extern uint8_t pwmDuty;
extern uint8_t pwmPin;
void coreinit();

#ifdef __cplusplus
}
#endif








#ifdef __cplusplus
extern "C" {
#endif
unsigned long rtc_get(void);
void rtc_set(unsigned long t);
void rtc_compensate(int adjust);
#ifdef __cplusplus
}
class teensy3_clock_class
{
public:
	static unsigned long get(void) __attribute__((always_inline)) { return rtc_get(); }
	static void set(unsigned long t) __attribute__((always_inline)) { rtc_set(t); }
	static void compensate(int adj) __attribute__((always_inline)) { rtc_compensate(adj); }
};
extern teensy3_clock_class Teensy3Clock;
#endif




#endif
