/* Teensyduino Core Library
 * http://www.pjrc.com/teensy/
 * Copyright (c) 2013 PJRC.COM, LLC.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * 1. The above copyright notice and this permission notice shall be 
 * included in all copies or substantial portions of the Software.
 *
 * 2. If the Software is incorporated into a build system that allows 
 * selection among a list of target devices, then similar target
 * devices manufactured by PJRC.COM must be included in the list of
 * target devices and selectable in the same manner.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include "core_pins.h"
#include "pins_arduino.h"
#include "HardwareSerial.h"

#include "FC1.h"

static uint8_t analog_write_res = 8;

// SOPT4 is SIM select clocks?
// FTM is clocked by the bus clock, either 24 or 48 MHz
// input capture can be FTM1_CH0, CMP0 or CMP1 or USB start of frame
// 24 MHz with reload 49152 to match Arduino's speed = 488.28125 Hz

void analogWrite(uint8_t pin, int val)
{
	if(pin > 1 && pin <= CORE_NUM_PWM){
		pwmPin=pin;
		if(val > 255) val=255;
		pinMode(pin, OUTPUT);    // sets the digital pin as output
		if(val == 0){
			digitalWrite(pin, 0);	// make pin low
		}
		pwmDuty=val;	// turn on pwm output for pin
	}


/*
	uint32_t cval, max;

#if defined(__MK20DX256__)
	if (pin == A14) {
		uint8_t res = analog_write_res;
		if (res < 12) {
			val <<= 12 - res;
		} else if (res > 12) {
			val >>= res - 12;
		}
		analogWriteDAC0(val);
		return;
	}
#elif defined(__MKL26Z64__)
	if (pin == A12) {
		uint8_t res = analog_write_res;
		if (res < 12) {
			val <<= 12 - res;
		} else if (res > 12) {
			val >>= res - 12;
		}
		analogWriteDAC0(val);
		return;
	}
#endif

	max = 1 << analog_write_res;
	if (val <= 0) {
		digitalWrite(pin, LOW);
		pinMode(pin, OUTPUT);	// TODO: implement OUTPUT_LOW
		return;
	} else if (val >= max) {
		digitalWrite(pin, HIGH);
		pinMode(pin, OUTPUT);	// TODO: implement OUTPUT_HIGH
		return;
	}

	//serial_print("analogWrite\n");
	//serial_print("val = ");
	//serial_phex32(val);
	//serial_print("\n");
	//serial_print("analog_write_res = ");
	//serial_phex(analog_write_res);
	//serial_print("\n");
	if (pin == FTM1_CH0_PIN || pin == FTM1_CH1_PIN) {
		cval = ((uint32_t)val * (uint32_t)(FTM1_MOD + 1)) >> analog_write_res;
#if defined(FTM2_CH0_PIN)
	} else if (pin == FTM2_CH0_PIN || pin == FTM2_CH1_PIN) {
		cval = ((uint32_t)val * (uint32_t)(FTM2_MOD + 1)) >> analog_write_res;
#endif
	} else {
		cval = ((uint32_t)val * (uint32_t)(FTM0_MOD + 1)) >> analog_write_res;
	}
	//serial_print("cval = ");
	//serial_phex32(cval);
	//serial_print("\n");
	switch (pin) {
#ifdef FTM0_CH0_PIN
	  case FTM0_CH0_PIN: // PTC1, FTM0_CH0
		FTM0_C0V = cval;
		FTM_PINCFG(FTM0_CH0_PIN) = PORT_PCR_MUX(4) | PORT_PCR_DSE | PORT_PCR_SRE;
		break;
#endif
#ifdef FTM0_CH1_PIN
	  case FTM0_CH1_PIN: // PTC2, FTM0_CH1
		FTM0_C1V = cval;
		FTM_PINCFG(FTM0_CH1_PIN) = PORT_PCR_MUX(4) | PORT_PCR_DSE | PORT_PCR_SRE;
		break;
#endif
#ifdef FTM0_CH2_PIN
	  case FTM0_CH2_PIN: // PTC3, FTM0_CH2
		FTM0_C2V = cval;
		FTM_PINCFG(FTM0_CH2_PIN) = PORT_PCR_MUX(4) | PORT_PCR_DSE | PORT_PCR_SRE;
		break;
#endif
#ifdef FTM0_CH3_PIN
	  case FTM0_CH3_PIN: // PTC4, FTM0_CH3
		FTM0_C3V = cval;
		FTM_PINCFG(FTM0_CH3_PIN) = PORT_PCR_MUX(4) | PORT_PCR_DSE | PORT_PCR_SRE;
		break;
#endif
#ifdef FTM0_CH4_PIN
	  case FTM0_CH4_PIN: // PTD4, FTM0_CH4
		FTM0_C4V = cval;
		FTM_PINCFG(FTM0_CH4_PIN) = PORT_PCR_MUX(4) | PORT_PCR_DSE | PORT_PCR_SRE;
		break;
#endif
#ifdef FTM0_CH5_PIN
	  case FTM0_CH5_PIN: // PTD5, FTM0_CH5
		FTM0_C5V = cval;
		FTM_PINCFG(FTM0_CH5_PIN) = PORT_PCR_MUX(4) | PORT_PCR_DSE | PORT_PCR_SRE;
		break;
#endif
#ifdef FTM0_CH6_PIN
	  case FTM0_CH6_PIN: // PTD6, FTM0_CH6
		FTM0_C6V = cval;
		FTM_PINCFG(FTM0_CH6_PIN) = PORT_PCR_MUX(4) | PORT_PCR_DSE | PORT_PCR_SRE;
		break;
#endif
#ifdef FTM0_CH7_PIN
	  case FTM0_CH7_PIN: // PTD7, FTM0_CH7
		FTM0_C7V = cval;
		FTM_PINCFG(FTM0_CH7_PIN) = PORT_PCR_MUX(4) | PORT_PCR_DSE | PORT_PCR_SRE;
		break;
#endif
#ifdef FTM1_CH0_PIN
	  case FTM1_CH0_PIN: // PTA12, FTM1_CH0
		FTM1_C0V = cval;
		FTM_PINCFG(FTM1_CH0_PIN) = PORT_PCR_MUX(3) | PORT_PCR_DSE | PORT_PCR_SRE;
		break;
#endif
#ifdef FTM1_CH1_PIN
	  case FTM1_CH1_PIN: // PTA13, FTM1_CH1
		FTM1_C1V = cval;
		FTM_PINCFG(FTM1_CH1_PIN) = PORT_PCR_MUX(3) | PORT_PCR_DSE | PORT_PCR_SRE;
		break;
#endif
#ifdef FTM2_CH0_PIN
	  case FTM2_CH0_PIN: // PTB18, FTM2_CH0
		FTM2_C0V = cval;
		FTM_PINCFG(FTM2_CH0_PIN) = PORT_PCR_MUX(3) | PORT_PCR_DSE | PORT_PCR_SRE;
		break;
#endif
#ifdef FTM2_CH1_PIN
	  case FTM2_CH1_PIN: // PTB19, FTM1_CH1
		FTM2_C1V = cval;
		FTM_PINCFG(FTM2_CH1_PIN) = PORT_PCR_MUX(3) | PORT_PCR_DSE | PORT_PCR_SRE;
		break;
#endif
	  default:
		digitalWrite(pin, (val > 127) ? HIGH : LOW);
		pinMode(pin, OUTPUT);
	}
*/
}


void analogWriteRes(uint32_t bits)
{
/*
	if (bits < 1) {
		bits = 1;
	} else if (bits > 16) {
		bits = 16;
	}
	analog_write_res = bits;

*/
}


void analogWriteFrequency(uint8_t pin, float frequency)
{
/*
	uint32_t prescale, mod;
	float minfreq;

	//serial_print("analogWriteFrequency: pin = ");
	//serial_phex(pin);
	//serial_print(", freq = ");
	//serial_phex32((uint32_t)frequency);
	//serial_print("\n");
	for (prescale = 0; prescale < 7; prescale++) {
		minfreq = (float)(F_TIMER >> prescale) / 65536.0f;
		if (frequency >= minfreq) break;
	}
	//serial_print("F_TIMER = ");
	//serial_phex32(F_TIMER >> prescale);
	//serial_print("\n");
	//serial_print("prescale = ");
	//serial_phex(prescale);
	//serial_print("\n");
	mod = (float)(F_TIMER >> prescale) / frequency - 0.5f;
	if (mod > 65535) mod = 65535;
	//serial_print("mod = ");
	//serial_phex32(mod);
	//serial_print("\n");
	if (pin == FTM1_CH0_PIN || pin == FTM1_CH1_PIN) {
		FTM1_SC = 0;
		FTM1_CNT = 0;
		FTM1_MOD = mod;
		FTM1_SC = FTM_SC_CLKS(1) | FTM_SC_PS(prescale);
	} else if (pin == FTM0_CH0_PIN || pin == FTM0_CH1_PIN
	  || pin == FTM0_CH2_PIN || pin == FTM0_CH3_PIN
	  || pin == FTM0_CH4_PIN || pin == FTM0_CH5_PIN
#ifdef FTM0_CH6_PIN
	  || pin == FTM0_CH6_PIN || pin == FTM0_CH7_PIN
#endif
	  ) {
		FTM0_SC = 0;
		FTM0_CNT = 0;
		FTM0_MOD = mod;
		FTM0_SC = FTM_SC_CLKS(1) | FTM_SC_PS(prescale);
	}
#ifdef FTM2_CH0_PIN
	  else if (pin == FTM2_CH0_PIN || pin == FTM2_CH1_PIN) {
		FTM2_SC = 0;
		FTM2_CNT = 0;
		FTM2_MOD = mod;
		FTM2_SC = FTM_SC_CLKS(1) | FTM_SC_PS(prescale);
	}
#endif
*/
}



// TODO: startup code needs to initialize all pins to GPIO mode, input by default

void digitalWriteFast(uint8_t pin, uint8_t val)
{
	if (pin == 0) {
	} else if (pin == 1) {
	} else if (pin == 2) {
		ShieldGPIOB_SetFieldValue(ShieldGPIOB_DeviceData, ShieldD2, val);
	} else if (pin == 3) {
		ShieldGPIOC_SetFieldValue(ShieldGPIOC_DeviceData, ShieldD3, val);
	} else if (pin == 4) {
		ShieldGPIOB_SetFieldValue(ShieldGPIOB_DeviceData, ShieldD4, val);
	} else if (pin == 5) {
		ShieldGPIOC_SetFieldValue(ShieldGPIOC_DeviceData, ShieldD5, val);
	} else if (pin == 6) {
		ShieldGPIOC_SetFieldValue(ShieldGPIOC_DeviceData, ShieldD6, val);
	} else if (pin == 7) {
		ShieldGPIOC_SetFieldValue(ShieldGPIOC_DeviceData, ShieldD7, val);
	} else if (pin == 8) {
		ShieldGPIOC_SetFieldValue(ShieldGPIOC_DeviceData, ShieldD8, val);
	} else if (pin == 9) {
		ShieldGPIOC_SetFieldValue(ShieldGPIOC_DeviceData, ShieldD9, val);
	} else if (pin == 10) {
		ShieldGPIOD_SetFieldValue(ShieldGPIOD_DeviceData, ShieldD10, val);
	} else if (pin == 11) {
		ShieldGPIOD_SetFieldValue(ShieldGPIOD_DeviceData, ShieldD11, val);
	} else if (pin == 12) {
		ShieldGPIOD_SetFieldValue(ShieldGPIOD_DeviceData, ShieldD12, val);
	} else if (pin == 13) {
		ShieldGPIOD_SetFieldValue(ShieldGPIOD_DeviceData, ShieldD13, val);
	}
}
void digitalWrite(uint8_t pin, uint8_t val)
{
	if(pin == pwmPin)	pwmDuty=0;	// turn off pwm output for pin
	digitalWriteFast(pin, val);
}

uint8_t digitalReadFast(uint8_t pin)
{
	if (pin == 0) {
	} else if (pin == 1) {
	} else if (pin == 2) {
		return ShieldGPIOB_GetFieldValue(ShieldGPIOB_DeviceData, ShieldD2);
	} else if (pin == 3) {
		return ShieldGPIOC_GetFieldValue(ShieldGPIOC_DeviceData, ShieldD3);
	} else if (pin == 4) {
		return ShieldGPIOB_GetFieldValue(ShieldGPIOB_DeviceData, ShieldD4);
	} else if (pin == 5) {
		return ShieldGPIOC_GetFieldValue(ShieldGPIOC_DeviceData, ShieldD5);
	} else if (pin == 6) {
		return ShieldGPIOC_GetFieldValue(ShieldGPIOC_DeviceData, ShieldD6);
	} else if (pin == 7) {
		return ShieldGPIOC_GetFieldValue(ShieldGPIOC_DeviceData, ShieldD7);
	} else if (pin == 8) {
		return ShieldGPIOC_GetFieldValue(ShieldGPIOC_DeviceData, ShieldD8);
	} else if (pin == 9) {
		return ShieldGPIOC_GetFieldValue(ShieldGPIOC_DeviceData, ShieldD9);
	} else if (pin == 10) {
		return ShieldGPIOD_GetFieldValue(ShieldGPIOD_DeviceData, ShieldD10);
	} else if (pin == 11) {
		return ShieldGPIOD_GetFieldValue(ShieldGPIOD_DeviceData, ShieldD11);
	} else if (pin == 12) {
		return ShieldGPIOD_GetFieldValue(ShieldGPIOD_DeviceData, ShieldD12);
	} else if (pin == 13) {
		return ShieldGPIOD_GetFieldValue(ShieldGPIOD_DeviceData, ShieldD13);
	}
}
uint8_t digitalRead(uint8_t pin)
{
	if(pin == pwmPin)	pwmDuty=0;	// turn off pwm output for pin
	return digitalReadFast(pin);
}



void pinMode(uint8_t pin, uint8_t mode)
{
	if(mode == INPUT || mode == INPUT_PULLUP){
		if(pin == pwmPin)	pwmDuty=0;	// turn off pwm output for pin
		if (pin == 0) {
		} else if (pin == 1) {
		} else if (pin == 2) {
			ShieldGPIOB_SetFieldInputDirection(ShieldGPIOB_DeviceData, ShieldD2);
		} else if (pin == 3) {
			ShieldGPIOC_SetFieldInputDirection(ShieldGPIOC_DeviceData, ShieldD3);
		} else if (pin == 4) {
			ShieldGPIOB_SetFieldInputDirection(ShieldGPIOB_DeviceData, ShieldD4);
		} else if (pin == 5) {
			ShieldGPIOC_SetFieldInputDirection(ShieldGPIOC_DeviceData, ShieldD5);
		} else if (pin == 6) {
			ShieldGPIOC_SetFieldInputDirection(ShieldGPIOC_DeviceData, ShieldD6);
		} else if (pin == 7) {
			ShieldGPIOC_SetFieldInputDirection(ShieldGPIOC_DeviceData, ShieldD7);
		} else if (pin == 8) {
			ShieldGPIOC_SetFieldInputDirection(ShieldGPIOC_DeviceData, ShieldD8);
		} else if (pin == 9) {
			ShieldGPIOC_SetFieldInputDirection(ShieldGPIOC_DeviceData, ShieldD9);
		} else if (pin == 10) {
			ShieldGPIOD_SetFieldInputDirection(ShieldGPIOD_DeviceData, ShieldD10);
		} else if (pin == 11) {
			ShieldGPIOD_SetFieldInputDirection(ShieldGPIOD_DeviceData, ShieldD11);
		} else if (pin == 12) {
			ShieldGPIOD_SetFieldInputDirection(ShieldGPIOD_DeviceData, ShieldD12);
		} else if (pin == 13) {
			ShieldGPIOD_SetFieldInputDirection(ShieldGPIOD_DeviceData, ShieldD13);
		}
		return;
	}
	if(mode == OUTPUT){
		if (pin == 0) {
		} else if (pin == 1) {
		} else if (pin == 2) {
			ShieldGPIOB_SetFieldOutputDirection(ShieldGPIOB_DeviceData, ShieldD2, 0);
		} else if (pin == 3) {
			ShieldGPIOC_SetFieldOutputDirection(ShieldGPIOC_DeviceData, ShieldD3, 0);
		} else if (pin == 4) {
			ShieldGPIOB_SetFieldOutputDirection(ShieldGPIOB_DeviceData, ShieldD4, 0);
		} else if (pin == 5) {
			ShieldGPIOC_SetFieldOutputDirection(ShieldGPIOC_DeviceData, ShieldD5, 0);
		} else if (pin == 6) {
			ShieldGPIOC_SetFieldOutputDirection(ShieldGPIOC_DeviceData, ShieldD6, 0);
		} else if (pin == 7) {
			ShieldGPIOC_SetFieldOutputDirection(ShieldGPIOC_DeviceData, ShieldD7, 0);
		} else if (pin == 8) {
			ShieldGPIOC_SetFieldOutputDirection(ShieldGPIOC_DeviceData, ShieldD8, 0);
		} else if (pin == 9) {
			ShieldGPIOC_SetFieldOutputDirection(ShieldGPIOC_DeviceData, ShieldD9, 0);
		} else if (pin == 10) {
			ShieldGPIOD_SetFieldOutputDirection(ShieldGPIOD_DeviceData, ShieldD10, 0);
		} else if (pin == 11) {
			ShieldGPIOD_SetFieldOutputDirection(ShieldGPIOD_DeviceData, ShieldD11, 0);
		} else if (pin == 12) {
			ShieldGPIOD_SetFieldOutputDirection(ShieldGPIOD_DeviceData, ShieldD12, 0);
		} else if (pin == 13) {
			ShieldGPIOD_SetFieldOutputDirection(ShieldGPIOD_DeviceData, ShieldD13, 0);
		}
		return;
	}
}


void _shiftOut(uint8_t dataPin, uint8_t clockPin, uint8_t bitOrder, uint8_t value)
{
        if (bitOrder == LSBFIRST) {
                shiftOut_lsbFirst(dataPin, clockPin, value);
        } else {
                shiftOut_msbFirst(dataPin, clockPin, value);
        }
}

void shiftOut_lsbFirst(uint8_t dataPin, uint8_t clockPin, uint8_t value)
{
        uint8_t mask;
        for (mask=0x01; mask; mask <<= 1) {
                digitalWrite(dataPin, value & mask);
                digitalWrite(clockPin, HIGH);
                digitalWrite(clockPin, LOW);
        }
}

void shiftOut_msbFirst(uint8_t dataPin, uint8_t clockPin, uint8_t value)
{
        uint8_t mask;
        for (mask=0x80; mask; mask >>= 1) {
                digitalWrite(dataPin, value & mask);
                digitalWrite(clockPin, HIGH);
                digitalWrite(clockPin, LOW);
        }
}

uint8_t _shiftIn(uint8_t dataPin, uint8_t clockPin, uint8_t bitOrder)
{
        if (bitOrder == LSBFIRST) {
                return shiftIn_lsbFirst(dataPin, clockPin);
        } else {
                return shiftIn_msbFirst(dataPin, clockPin);
        }
}

uint8_t shiftIn_lsbFirst(uint8_t dataPin, uint8_t clockPin)
{
        uint8_t mask, value=0;
        for (mask=0x01; mask; mask <<= 1) {
                digitalWrite(clockPin, HIGH);
                if (digitalRead(dataPin)) value |= mask;
                digitalWrite(clockPin, LOW);
        }
        return value;
}

uint8_t shiftIn_msbFirst(uint8_t dataPin, uint8_t clockPin)
{
        uint8_t mask, value=0;
        for (mask=0x80; mask; mask >>= 1) {
                digitalWrite(clockPin, HIGH);
                if (digitalRead(dataPin)) value |= mask;
                digitalWrite(clockPin, LOW);
        }
        return value;
}



// the systick interrupt is supposed to increment this at 1 kHz rate
volatile uint32_t systick_millis_count = 0;
volatile uint32_t delay_millis_count = 0;

//uint32_t systick_current, systick_count, systick_istatus;  // testing only

uint32_t micros(void)
{
	uint32_t count, current;

	count = FC1_GetCounterValue(FC1_DeviceData);
	// adjust for 1.31072 MHz, counter increments every 0.762939 uS
	current = count * 0.76294;
	return (systick_millis_count * 1000) + current;

/*
	uint32_t count, current, istatus;

	__disable_irq();
	current = SYST_CVR;
	count = systick_millis_count;
	istatus = SCB_ICSR;	// bit 26 indicates if systick exception pending
	__enable_irq();
	 //systick_current = current;
	 //systick_count = count;
	 //systick_istatus = istatus & SCB_ICSR_PENDSTSET ? 1 : 0;
	if ((istatus & SCB_ICSR_PENDSTSET) && current > 50) count++;
	current = ((F_CPU / 1000) - 1) - current;
#if defined(KINETISL) && F_CPU == 48000000
	return count * 1000 + ((current * (uint32_t)87381) >> 22);
#elif defined(KINETISL) && F_CPU == 24000000
	return count * 1000 + ((current * (uint32_t)174763) >> 22);
#endif
	return count * 1000 + current / (F_CPU / 1000000);
*/
}

// delay in microseconds
void delayMicroseconds(uint32_t usec){

	uint32_t start;

	if (usec > 0) {
		start = micros();
		while((micros() - start) < usec) ;
	}

}

// delay in milliseconds
void delay(uint32_t ms)
{
	delay_millis_count=0;
	while(delay_millis_count < ms) ;
/*

	uint32_t start = micros();

	if (ms > 0) {
		while (1) {
			if ((micros() - start) >= 1000) {
				ms--;
				if (ms == 0) return;
				start += 1000;
			}
		}
	}
*/
}

// TODO: verify these result in correct timeouts...
#if F_CPU == 168000000
#define PULSEIN_LOOPS_PER_USEC 25
#elif F_CPU == 144000000
#define PULSEIN_LOOPS_PER_USEC 21
#elif F_CPU == 120000000
#define PULSEIN_LOOPS_PER_USEC 18
#elif F_CPU == 96000000
#define PULSEIN_LOOPS_PER_USEC 14
#elif F_CPU == 72000000
#define PULSEIN_LOOPS_PER_USEC 10
#elif F_CPU == 48000000
#define PULSEIN_LOOPS_PER_USEC 7
#elif F_CPU == 24000000
#define PULSEIN_LOOPS_PER_USEC 4
#elif F_CPU == 16000000
#define PULSEIN_LOOPS_PER_USEC 1
#elif F_CPU == 8000000
#define PULSEIN_LOOPS_PER_USEC 1
#elif F_CPU == 4000000
#define PULSEIN_LOOPS_PER_USEC 1
#elif F_CPU == 2000000
#define PULSEIN_LOOPS_PER_USEC 1
#endif


uint32_t pulseIn_high(volatile uint8_t *reg, uint32_t timeout)
{
	uint32_t timeout_count = timeout * PULSEIN_LOOPS_PER_USEC;
	uint32_t usec_start, usec_stop;
	
	// wait for any previous pulse to end
	while (*reg) {
		if (--timeout_count == 0) return 0;
	}
	// wait for the pulse to start
	while (!*reg) {
		if (--timeout_count == 0) return 0;
	}
	usec_start = micros();
	// wait for the pulse to stop
	while (*reg) {
		if (--timeout_count == 0) return 0;
	}
	usec_stop = micros();
	return usec_stop - usec_start;
}

uint32_t pulseIn_low(volatile uint8_t *reg, uint32_t timeout)
{
	uint32_t timeout_count = timeout * PULSEIN_LOOPS_PER_USEC;
	uint32_t usec_start, usec_stop;
	
	// wait for any previous pulse to end
	while (!*reg) {
		if (--timeout_count == 0) return 0;
	}
	// wait for the pulse to start
	while (*reg) {
		if (--timeout_count == 0) return 0;
	}
	usec_start = micros();
	// wait for the pulse to stop
	while (!*reg) {
		if (--timeout_count == 0) return 0;
	}
	usec_stop = micros();
	return usec_stop - usec_start;
}

// TODO: an inline version should handle the common case where state is const
uint32_t pulseIn(uint8_t pin, uint8_t state, uint32_t timeout)
{
	if (pin >= CORE_NUM_DIGITAL) return 0;
	if (state) return pulseIn_high(portInputRegister(pin), timeout);
	return pulseIn_low(portInputRegister(pin), timeout);;
}


void coreinit(){

	// turn off all pwm outputs
	pwmDuty=0;
}
