/*
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include "AS1.h"
#include "HardwareSerial.h"

LDD_TDeviceData *Uart1Ptr = NULL;	// pointer to UART 1 component AS1


static volatile uint8_t transmitting = 0;

void serial_begin(uint32_t divisor)
{
	LDD_SERIAL_TBaudMode	bmode;

	if(Uart1Ptr == NULL)	Uart1Ptr = AS1_Init(NULL);	// Initialize UART 1 component AS1
	// set baud rate
	switch(divisor){
		case 256000: bmode=AS1_BM_256000BAUD; break;
		case 115200: bmode=AS1_BM_115200BAUD; break;
		case 57600:  bmode=AS1_BM_57600BAUD; break;
		case 38400:  bmode=AS1_BM_38400BAUD; break;
		case 19200:  bmode=AS1_BM_19200BAUD; break;
		case 14400:  bmode=AS1_BM_14400BAUD; break;
		case 9600:   bmode=AS1_BM_9600BAUD; break;
		case 4800:   bmode=AS1_BM_4800BAUD; break;
		case 2400:   bmode=AS1_BM_2400BAUD; break;
		case 1200:   bmode=AS1_BM_1200BAUD; break;
		case 600:    bmode=AS1_BM_600BAUD; break;
		default:
			bmode=AS1_BM_115200BAUD; break;	// invalid baud, use default
	}
	AS1_SelectBaudRate(Uart1Ptr, bmode);
}

void serial_format(uint32_t format)
{
	/*
        uint8_t c;

        c = UART0_C1;
        c = (c & ~0x13) | (format & 0x03);      // configure parity
        if (format & 0x04) c |= 0x10;           // 9 bits (might include parity)
        UART0_C1 = c;
        if ((format & 0x0F) == 0x04) UART0_C3 |= 0x40; // 8N2 is 9 bit with 9th bit always 1
        c = UART0_S2 & ~0x10;
        if (format & 0x10) c |= 0x10;           // rx invert
        UART0_S2 = c;
        c = UART0_C3 & ~0x10;
        if (format & 0x20) c |= 0x10;           // tx invert
        UART0_C3 = c;
#ifdef SERIAL_9BIT_SUPPORT
        c = UART0_C4 & 0x1F;
        if (format & 0x08) c |= 0x20;           // 9 bit mode with parity (requires 10 bits)
        UART0_C4 = c;
        use9Bits = format & 0x80;
#endif
*/
}

void serial_end(void)
{
/*
	if (!(SIM_SCGC4 & SIM_SCGC4_UART0_MASK)) return;
//	while (transmitting) yield();  // wait for buffered data to send
	while (transmitting) ;  // wait for buffered data to send
	NVIC_DISABLE_IRQ(IRQ_UART0_STATUS);
	UART0_C2 = 0;
	CORE_PIN0_CONFIG = PORT_PCR_PE_MASK | PORT_PCR_PS_MASK | PORT_PCR_MUX(1);
	CORE_PIN1_CONFIG = PORT_PCR_PE_MASK | PORT_PCR_PS_MASK | PORT_PCR_MUX(1);
	rx_buffer_head = 0;
	rx_buffer_tail = 0;
*/
}

void serial_set_transmit_pin(uint8_t pin)
{
/*
	while (transmitting) ;
	pinMode(pin, OUTPUT);
	digitalWrite(pin, LOW);
	transmit_pin = portOutputRegister(pin);
	#if defined(KINETISL)
	transmit_mask = digitalPinToBitMask(pin);
	#endif
*/
}

void serial_putchar(uint32_t c)
{
	if(AS1_SendBlock(Uart1Ptr, &c, 1) == ERR_OK){
		while(((AS1_TDeviceDataPtr)Uart1Ptr)->OutDataNumReq != 0x00U) ;	// wait for transmit complete
	}
}

void serial_write(const void *buf, unsigned int count)
{
	if(AS1_SendBlock(Uart1Ptr, buf, count) == ERR_OK){
		while(((AS1_TDeviceDataPtr)Uart1Ptr)->OutDataNumReq != 0x00U) ;	// wait for transmit complete
	}
}

void serial_flush(void)
{
	if(((AS1_TDeviceDataPtr)Uart1Ptr)->OutDataNumReq != 0){	// if data still being sent
		while(!AS1_GetTxCompleteStatus(Uart1Ptr)) ; // wait for transmit complete
	}
}

int serial_write_buffer_free(void)
{
/*
	uint32_t head, tail;

	head = tx_buffer_head;
	tail = tx_buffer_tail;
	if (head >= tail) return TX_BUFFER_SIZE - 1 - head + tail;
	return tail - head - 1;
*/
}

int serial_available(void)
{
/*
	uint32_t head, tail;

	head = rx_buffer_head;
	tail = rx_buffer_tail;
	if (head >= tail) return head - tail;
	return RX_BUFFER_SIZE + head - tail;
*/
}

int serial_getchar(void)
{
	int c;
	AS1_ReceiveBlock(Uart1Ptr, &c, 1);
	while(AS1_GetReceivedDataNum(Uart1Ptr) < 1) ;	// wait for data to be received
	return c;
}

int serial_peek(void)
{
/*
	uint32_t head, tail;

	head = rx_buffer_head;
	tail = rx_buffer_tail;
	if (head == tail) return -1;
	if (++tail >= RX_BUFFER_SIZE) tail = 0;
	return rx_buffer[tail];
*/
}

void serial_clear(void)
{
	AS1_CancelBlockTransmission(Uart1Ptr);
	AS1_CancelBlockReception(Uart1Ptr);
}

void serial_print(const char *p)
{
	serial_write(p, strlen(p));
}

static void serial_phex1(uint32_t n)
{
	n &= 15;
	if (n < 10) {
		serial_putchar('0' + n);
	} else {
		serial_putchar('A' - 10 + n);
	}
}

void serial_phex(uint32_t n)
{
	serial_phex1(n >> 4);
	serial_phex1(n);
}

void serial_phex16(uint32_t n)
{
	serial_phex(n >> 8);
	serial_phex(n);
}

void serial_phex32(uint32_t n)
{
	serial_phex(n >> 24);
	serial_phex(n >> 16);
	serial_phex(n >> 8);
	serial_phex(n);
}

